package com.cs427.swordfish.banking;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

/*
 * ATM is the actor to handle ATM transaction
 * It takes a Bank Actor in the constructor and then delegates its functionality to the bank
 */
public class ATM extends Actor {

	// Serializable ID
	private static final long serialVersionUID = 1L;

	// Bank object to delegate transactions too
	private ActorName bank;

	/*
	 * Constructor
	 * 
	 * @param bank bank object to delegate transactions too
	 */
	public ATM(ActorName bank) throws RemoteCodeException {
		this.bank = bank;
	}

	/*
	 * withdraws amount from account
	 * 
	 * @param trans holds all relevant transaction information
	 */
	@message
	public void withdraw(Transaction trans) throws RemoteCodeException {
		send(bank, "withdraw", trans);
	}

	/*
	 * deposits amount from account
	 * 
	 * @param trans holds all relevant transaction information
	 */
	@message
	public void deposit(Transaction trans) throws RemoteCodeException {
		send(bank, "deposit", trans);
	}

	/*
	 * transfer amount from one account to another
	 * 
	 * @param trans holds all relevant transaction information
	 */
	@message
	public void transfer(Transfer trans) throws RemoteCodeException {
		send(bank, "transfer", trans);
	}

	/*
	 * gets the balance from account
	 * 
	 * @param trans holds all relevant transaction information
	 */
	@message
	public void balanceInquiry(Transaction trans) throws RemoteCodeException {
		send(bank, "balanceInquiry", trans);
	}
}
