package com.cs427.swordfish.banking;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;
//the following is commented due to eclipse project stuff w/ basset
//import gov.nasa.jpf.actor.core.PlatformUtil;

public class Driver extends Actor {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/* This is commented out because basset isn't present in the eclipse project
	public static void main(String[] args) {
		ActorName driver = PlatformUtil.createActor(Driver.class);
		PlatformUtil.send(driver, "run");
	}*/

	@message
	public void run() throws RemoteCodeException {
		System.out.println("Beginning Tests\n");
		ActorName bank = create(BankActor.class);

		/*
		 * Testing Create Account Should print to standard out: Account opened.
		 */
		System.out.println("Create account tests...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 10000, "1234", 1, 1.0));
		System.out.println("Create test done");
		System.out.println();

		/*
		 * Testing Close Account Should print to standard out: Account not
		 * closed: nonzero balance. No such account. Account opened. Account
		 * closed. No such account.
		 */
		System.out.println("Close account testing");
		// try to close non-zero balanced account
		call(bank, "closeAccount", new Transaction(create(this.getClass()),
				"createCallback", 1, 0, "1234", 1, 1.0));
		// try to close non-existent account
		call(bank, "closeAccount", new Transaction(create(this.getClass()),
				"createCallback", 2, 0, "1234", 1, 1.0));
		// open a new account with zero-balance
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 0, "1234", 0, 1.0));
		// correctly close account
		call(bank, "closeAccount", new Transaction(create(this.getClass()),
				"createCallback", 2, 0, "1234", 1, 1.0));
		// try to close already closed account
		call(bank, "closeAccount", new Transaction(create(this.getClass()),
				"createCallback", 2, 0, "1234", 1, 1.0));
		System.out.println("Close testing done");
		System.out.println();

		/*
		 * Testing Deposit Should print to standard out: Account opened. Deposit
		 * succeeded. No such account
		 */
		System.out.println("deposit testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 0, "1234", 1, 1.0));
		// deposit into account 3
		call(bank, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 3, 2500, "1234", 1, 1.0));
		// deposit into non existent account 15
		call(bank, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 15, 1000, "1234", 1, 1.0));
		System.out.println("deposit test done");
		System.out.println();

		/*
		 * Testing Withdraw Should print to standard out: Withdraw succeeded. No
		 * such account
		 */
		// withdraw from account 3
		System.out.println("withdraw testing...");
		call(bank, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 3, 2000, "1234", 1, 1.0));
		// withdraw from non existent account 12
		call(bank, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 12, 500, "1234", 1, 1.0));
		System.out.println("withdraw testing done");
		System.out.println();

		/*
		 * Testing transfer Should print to standard out: Account opened.
		 * Transfer succeeded. No such account: 20 No such account: 30
		 */
		System.out.println("transfer testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 4000, "1234", 1, 1.0));
		// do transfer from account 4 to 3
		call(bank, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 4, 3, 2000, "1234"));
		// do transfer from non existent account 20
		call(bank, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 20, 3, 200, "1234"));
		// do transfer to non existent account 30
		call(bank, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 4, 30, 200, "1234"));
		System.out.println("transfer test done");
		System.out.println();

		/*
		 * Testing get Balance Should print to standard out: Account opened.
		 * Here is your balance: 2500 Here is your balance: 2500 No such
		 * account: 10
		 */

		System.out.println("get balance tests...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 2500, "1234", 1, 1.0));
		// do balance inquiry on account 5
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 5, 0, "1234", 1, 1.0));
		// do balance inquiry on account 3
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 3, 0, "1234", 1, 1.0));
		// do balance inquiry on non existent account 10
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 10, 0, "1234", 1, 1.0));
		System.out.println("get balance tests done");
		System.out.println();

		/*
		 * Testing overdraw on withdrawal Should print to standard out: Withdraw
		 * failed.
		 */
		System.out.println("withdraw overdraft tests...");
		call(bank, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 5, 6000, "1234", 1, 1.0));
		System.out.println("withdraw overdraft done...");
		System.out.println();

		/*
		 * Testing Pin verification Should print to standard out: Account
		 * opened. Deposit succeeded. Invalid PIN Here is your balance: 2500
		 */
		System.out.println("PIN testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 0, "1234", 1, 1.0));
		// deposit into account 6 with right pin
		call(bank, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 6, 2500, "1234", 1, 1.0));
		// deposit into account 6 with wrong pin
		call(bank, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 6, 1000, "1235", 1, 1.0));
		// do balance inquiry on account 6
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 6, 0, "1234", 1, 1.0));
		System.out.println("PIN test done");
		System.out.println();

		/*
		 * Testing withdraw multiple of 20 verification Should print to standard
		 * out: Account opened. Deposit succeeded Withdraw failed. Withdraw
		 * succeeded.
		 */
		System.out.println("withdraw amount multiple testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 0, "1234", 1, 1.0));
		// deposit into account 7
		call(bank, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 7, 3000, "1234", 1, 1.0));
		// withdraw amount not multiple of 20
		call(bank, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 7, 1500, "1234", 1, 1.0));
		// withdraw amount multiple of 20
		call(bank, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 7, 2000, "1234", 1, 1.0));
		System.out.println("withdraw amount multiple test done");
		System.out.println();

		/*
		 * Testing transfer overdraft Should print to standard out: Account
		 * opened. Account opened Transfer failed. Sorry. Your Transfer
		 * succeeded Here is your balance: 2000 Here is your balance: 6000
		 */

		System.out.println("transfer overdraft testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 4000, "1234", 1, 1.0));
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 4000, "1234", 1, 1.0));
		// do transfer from account 8 to 9 with overdraft
		call(bank, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 8, 9, 6000, "1234"));
		// do transfer from 8 to 9 without overdraft
		call(bank, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 8, 9, 2000, "1234"));
		// do balance inquiry on 8
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 8, 0, "1234", 1, 1.0));
		// do balance inquiry on 9
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 9, 0, "1234", 1, 1.0));
		System.out.println("transfer test done");
		System.out.println();

		/*
		 * Testing lock/unlock accounts Should print to standard out: No such
		 * account:12 No such account:12 Account opened. Account successfully
		 * locked! Account is locked. Account successfully unlocked! Here is
		 * your balance: 10000
		 */
		System.out.println("lock unlock tests...");
		// locking non existent account
		call(bank, "lockAccount", new Transaction(create(this.getClass()),
				"createCallback", 12, 10000, "1234", 1, 1.0));
		// unlocking non existent account
		call(bank, "unlockAccount", new Transaction(create(this.getClass()),
				"createCallback", 12, 10000, "1234", 1, 1.0));
		// create account number 10
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 10000, "1234", 1, 1.0));
		// lock account 10
		call(bank, "lockAccount", new Transaction(create(this.getClass()),
				"createCallback", 10, 10000, "1234", 1, 1.0));
		// check if we can do transactions on a locked account
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 10, 0, "1234", 1, 1.0));
		// unlock account 10
		call(bank, "unlockAccount", new Transaction(create(this.getClass()),
				"createCallback", 10, 10000, "1234", 1, 1.0));
		// do transactions on account 10 after unlocking it
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 10, 0, "1234", 1, 1.0));

		System.out.println("lock/unlock test done");
		System.out.println();

		/*
		 * Testing Burglar protection Should print to standard out: Account
		 * opened. Invalid PIN. Invalid PIN. Invalid PIN. Account is locked.
		 */
		System.out.println("Burglar protection tests ...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 10000, "1234", 1, 1.0));
		// do balance inquiry 3 times with wrong pin
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 11, 0, "1235", 1, 1.0));
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 11, 0, "1235", 1, 1.0));
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 11, 0, "1235", 1, 1.0));
		// do it once more after account is locked
		call(bank, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 11, 0, "1234", 1, 1.0));
		System.out.println("Burglar protection test done");
		System.out.println();

		ActorName atm = create(ATM.class, bank);
		/*
		 * Testing ATM Deposit Should print to standard out: Account opened.
		 * Deposit succeeded. No such account
		 */
		System.out.println("ATM deposit testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 0, "1234", 1, 1.0));
		// deposit into account 3
		call(atm, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 12, 2500, "1234", 1, 1.0));
		// deposit into non existent account 15
		call(atm, "deposit", new Transaction(create(this.getClass()),
				"createCallback", 15, 1000, "1234", 1, 1.0));
		System.out.println("ATM deposit test done");
		System.out.println();

		/*
		 * Testing ATM Withdraw Should print to standard out: Withdraw
		 * succeeded. No such account
		 */
		// withdraw from account 3
		System.out.println("ATM withdraw testing...");
		call(atm, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 12, 2000, "1234", 1, 1.0));
		// withdraw from non existent account 12
		call(atm, "withdraw", new Transaction(create(this.getClass()),
				"createCallback", 13, 500, "1234", 1, 1.0));
		System.out.println("ATM withdraw testing done");
		System.out.println();

		/*
		 * Testing ATM transfer Should print to standard out: Account opened.
		 * Transfer succeeded. No such account: 20 No such account: 30
		 */
		System.out.println("ATM transfer testing...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 4000, "1234", 1, 1.0));
		// do transfer from account 4 to 3
		call(atm, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 13, 12, 2000, "1234"));
		// do transfer from non existent account 20
		call(atm, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 20, 12, 200, "1234"));
		// do transfer to non existent account 30
		call(atm, "transfer", new Transfer(create(this.getClass()),
				"createCallback", 13, 30, 200, "1234"));
		System.out.println("ATM transfer test done");
		System.out.println();

		/*
		 * Testing ATM get Balance Should print to standard out: Account opened.
		 * Here is your balance: 2500 Here is your balance: 2500 No such
		 * account: 20
		 */
		System.out.println("ATM get balance tests...");
		call(bank, "createAccount", new Transaction(create(this.getClass()),
				"createCallback", null, 2500, "1234", 1, 1.0));
		// do balance inquiry on account 5
		call(atm, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 14, 0, "1234", 1, 1.0));
		// do balance inquiry on account 3
		call(atm, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 12, 0, "1234", 1, 1.0));
		// do balance inquiry on non existent account 10
		call(atm, "balanceInquiry", new Transaction(create(this.getClass()),
				"createCallback", 20, 0, "1234", 1, 1.0));
		System.out.println("ATM get balance tests done");
		System.out.println();
		/*
		 * End ATM Tests
		 */
	}

	@message
	public void createCallback(Response resp) {
		System.out.println(resp.getMessage());
	}
}
