package com.cs427.swordfish.banking;

import java.io.Serializable;

/*
 * Encapsulating abstract object for sending responses to the client
 */
public abstract class Response implements Serializable {

	// Serializable ID
	private static final long serialVersionUID = 1L;

	// Message to be returned
	private String message;

	/*
	 * Constructor
	 * 
	 * @param message Message to be returned
	 */
	public Response(String message) {
		this.message = message;
	}

	/*
	 * Getter for message
	 * 
	 * @return message
	 */
	public String getMessage() {
		return message;
	}
}

/*
 * Derived Response object for successful transaction
 */
class SuccessResponse extends Response {

	// Serializable ID
	private static final long serialVersionUID = 1L;

	/*
	 * Constructor
	 * 
	 * @param message Message to be sent to client
	 */
	public SuccessResponse(String message) {
		super(message);
	}
}

/*
 * Derived Response object for GetBalance transactions
 */
class BalanceResponse extends SuccessResponse {

	// Serializable ID
	private static final long serialVersionUID = 1L;

	// Amount in balance in cents
	private double cents;

	/*
	 * Constructor
	 * 
	 * @param cents cents in account
	 */
	public BalanceResponse(double cents) {
		super("Here is your balance: " + cents);
		this.cents = cents;
	}

	/*
	 * Getter for cents
	 * 
	 * @return cents
	 */
	public double getCents() {
		return cents;
	}
}

/*
 * Derived Response object for Opening an account
 */
class OpenedResponse extends SuccessResponse {

	// Serializable ID
	private static final long serialVersionUID = 1L;

	// Account number for opened account
	private Integer accountNumber;

	/*
	 * Constructor
	 * 
	 * @param acctNumber Account Number
	 */
	public OpenedResponse(Integer acctNumber) {
		super("Account opened.");
		accountNumber = acctNumber;
	}

	/*
	 * Getter for account number
	 * 
	 * @return Account number
	 */
	public Integer getAccountNumber() {
		return accountNumber;
	}
}

/*
 * Derived Response object for failed transactions
 */
class FailureResponse extends Response {

	// Serializable ID
	private static final long serialVersionUID = 1L;

	/*
	 * Constructor
	 * 
	 * @param message Message to be returned to client
	 */
	public FailureResponse(String message) {
		super(message);
	}
}
