package edu.illinois.cs.cs427.clownfish.actorBanking;
import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

public class ActorTest extends Actor {
	
	
	@message
	private ActorName initStdTest(String testName)
	{
		printTestName(testName);
		
		try	{
			ActorName context = (ActorName)create(TestContext.class);
			return context;
		} catch (RemoteCodeException e) {
			printTestResult(testName, false);
			
			return null;
		}
	}

	private static void printTestName(String testName)
	{
		System.out.println("Starting Test: " + testName);
	}
	
	private static void printTestResult(String testName, boolean result)
	{
		if (result)
		{
			System.out.println(testName + ": Succeeded!");
		}
		else
		{
			System.err.println(testName + ": Failed!");
		}
		System.out.println();
	}
	
	/**
	 * Runs a series of tests on Clownfish Banking code
	 * Test names must be added manually
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testAll() throws RemoteCodeException {
		boolean contextDepenenciesWorking = true;
		//ensures that objects can be initialized
		contextDepenenciesWorking &= testBankInit();
		contextDepenenciesWorking &= testATMInit();
		contextDepenenciesWorking &= testOpenAccount();
		
		if (contextDepenenciesWorking) {
			boolean allTestsWorking = true;
			
			//add test names here			
			allTestsWorking &= testBalanceInquiry();
			allTestsWorking &= testDeposit();
			allTestsWorking &= testWithdraw();
			allTestsWorking &= testOverdraw();
			allTestsWorking &= testTransfer();
			allTestsWorking &= testPrintLog();
			allTestsWorking &= testDepositTransactionFee();
			allTestsWorking &= testWithdrawTransactionFee();
			allTestsWorking &= testIncorrectPinCheck();
			allTestsWorking &= testPinSetting();
			
			//End of tests
			System.out.println("End of tests reached.");
			System.out.println("Did all tests pass?  " + (allTestsWorking ? "yes" : "no"));
		}
		else {
			System.err.println("Core Functionality Not Working! Skipping remaining tests!");
		}
	}
	
	/**
	 * Tests that bank objects can be initialized
	 * 
	 * @return a boolean signifying whether a bank has been initialized
	 * @throws RemoteCodeException
	 */
	
	@message
	public Boolean testBankInit() throws RemoteCodeException {
		ActorName theBank = create(ActorBank.class, 0);
		call(theBank, "initialize");
		
		printTestResult("Bank Init", true);
		return true;
	}
	
	
	/**
	 * Tests that ATM objects can be initialized
	 * 
	 * @return a boolean signifying whether an ATM has been initialized, always true unless code breaks.
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testATMInit() throws RemoteCodeException {
		ActorName theBank = create(ActorBank.class);
		call(theBank, "initialize");
		ActorName theATM = create(ActorATM.class, theBank);

		printTestResult("ATM Init", true);
		return true;
	}

	/**
	 * Tests whether accounts can be successfully opened
	 * 
	 * @return a boolean signifying whether an account can be opened, always true unless code breaks.
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testOpenAccount() throws RemoteCodeException {
		ActorName theBank = create(ActorBank.class);
		call(theBank, "initialize");
		ActorName theATM = create(ActorATM.class, theBank);
	
		//Testing Open Account
		Integer acctID = (Integer) call(theATM, "openAccount", 100, 1111);
		printTestResult("Open Account", true);
		return true;
	}
	
	
	/**
	 * Tests whether an account's balance can be inquired correctly
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testBalanceInquiry() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Balance Inquiry");
		call(context, "getNewTestContext");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");

		final int expectedBalance = TestContext.initialBalance1;
		
		//Testing Balance Inquiry
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);
		boolean success = (bankingResponse instanceof SuccessResponse) &&
			((BalanceResponse) bankingResponse).getBalance().equals(expectedBalance);
		printTestResult("Balance Inquiry", success);
		
		return success;
	}
	
	/**
	 * Tests that money can correctly be deposited in an account
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testDeposit() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Deposit");
		call(context, "getNewTestContext");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");

		final int depositAmount = 100;
		final int expectedBalance = TestContext.initialBalance1 + depositAmount;
		
		//Testing Deposit
		BankingResponse depositResponse = (BankingResponse) call(theATM, "deposit", acctID, depositAmount, acctPin);
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);

		boolean success = (depositResponse instanceof SuccessResponse) &&
			((BalanceResponse) bankingResponse).getBalance().equals(expectedBalance);
	
		printTestResult("Deposit", success);
		
		return success;
	}

	/**
	 * Tests that money can be successfully withdrawn from an account
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testWithdraw() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Withdraw");
		call(context, "getNewTestContext");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");

		final int withdrawAmonut = 50;
		int expectedBalance = TestContext.initialBalance1 - withdrawAmonut;
		
		//Testing Withdraw
		BankingResponse withdrawResponse = (BankingResponse) call(theATM, "withdraw", acctID, withdrawAmonut, acctPin);
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);
		
		boolean success = (withdrawResponse instanceof SuccessResponse) &&
			((BalanceResponse) bankingResponse).getBalance().equals(expectedBalance);
		
		printTestResult("Withdraw", success);
		return success;
	}
		
	
	/**
	 * Tests that overdrawing causes a withdrawal to fail
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testOverdraw() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Overdraw");
		call(context, "getNewTestContext");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");

		final int egregiousAmount = Integer.MAX_VALUE;
		final int expectedAmount = TestContext.initialBalance1;
		
		//Testing Overdraw
		BankingResponse overdrawResponse = (BankingResponse) call(theATM, "withdraw", acctID, egregiousAmount, acctPin);
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);
		
		boolean success = (overdrawResponse instanceof FailureResponse) &&
			((BalanceResponse) bankingResponse).getBalance().equals(expectedAmount);
			
		printTestResult("Overdraw", success);
		return success;
	}
	
	
	/**
	 * Tests that money can be transferred from one account to another
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testTransfer() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Transfer");
		call(context, "getNewTestContext");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");
		Integer acctIDTwo = (Integer)call(context, "getSecondAccount");
		Integer acctPinTwo = (Integer)call(context, "getSecondPin");

		final int transferAmount = 50;
		final int expectedAmount1 = TestContext.initialBalance1 - transferAmount;
		final int expectedAmount2 = TestContext.initialBalance2 + transferAmount;
		
		//Testing Transfer
		BankingResponse transferResponse = (BankingResponse) call(theATM, "transfer", acctID, acctIDTwo, transferAmount, acctPin);
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);
		BankingResponse balanceResponseTwo = (BankingResponse) call(theATM, "balanceInquiry", acctIDTwo, acctPinTwo);
		
		boolean success = (transferResponse instanceof SuccessResponse) && 
			((BalanceResponse) bankingResponse).getBalance().equals(expectedAmount1) &&
			((BalanceResponse) balanceResponseTwo).getBalance().equals(expectedAmount2);

		printTestResult("Transfer", success);
		return success;
	}
	
	
	/**
	 * Tests that a log of account activity can be printed
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testPrintLog() throws RemoteCodeException {		
		ActorName context = (ActorName)initStdTest("Print Log");
		call(context, "getNewTestContext");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");
		BankingResponse printResponse = (BankingResponse) call(theATM, "printLog", acctID, acctPin);
	
		//We assume it always works...
		printTestResult("Print Log", true);
		return true;
	}
	
	
	/**
	 * Tests that a transaction fee can be applied to a deposit
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testDepositTransactionFee() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Deposit Transaction Fee");
		call(context, "getNewTestContext");
		ActorName theBank = (ActorName)call(context,"getBank");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");
		final int depositAmount = 100;
		final int feeAmount = 2;
		final int expectedAmount = (TestContext.initialBalance1 + depositAmount) - feeAmount;
		
		//Testing Modify
		call(theBank, "modifyTransactionFee", feeAmount);
		BankingResponse depositResponse = (BankingResponse) call(theATM, "deposit", acctID, depositAmount, acctPin);
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);
		
		boolean success = (depositResponse instanceof SuccessResponse) && 
			((BalanceResponse) bankingResponse).getBalance().equals(expectedAmount);

		printTestResult("Deposit Transaction Fee", success);
		return success;
	}
	
	/**
	 * Tests that a transaction fee can be applied to a withdrawal
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testWithdrawTransactionFee() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Withdraw Transaction Fee");
		call(context, "getNewTestContext");
		ActorName theBank = (ActorName)call(context,"getBank");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin");
		final int withdrawAmount = 46;
		final int feeAmount = 2;
		final int expectedAmount = (TestContext.initialBalance1 - withdrawAmount) - feeAmount;
		
		call(theBank, "modifyTransactionFee", feeAmount);
		BankingResponse withdrawResponse = (BankingResponse) call(theATM, "withdraw", acctID, withdrawAmount, acctPin);
		BankingResponse bankingResponse = (BankingResponse) call(theATM, "balanceInquiry", acctID, acctPin);
		
		boolean success = (withdrawResponse instanceof SuccessResponse) && 
			((BalanceResponse) bankingResponse).getBalance().equals(expectedAmount);
		
		printTestResult("Withdraw Transaction Fee", success);
		return success;
	}
	
	
	/**
	 * Tests that using an incorrect pin causes denied access
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testIncorrectPinCheck() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Incorrect Pin Checking");
		call(context, "getNewTestContext");
		ActorName theBank = (ActorName)call(context,"getBank");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer badPin = (Integer)call(context, "getSecondPin"); 
			
		final int withdrawAmount = 40;
		
		BankingResponse badPinResponse = (BankingResponse) call(theATM, "withdraw", acctID, withdrawAmount, badPin);

		boolean success = (badPinResponse instanceof DenialResponse);
	
		printTestResult("Incorrect Pin Checking", success);
		return success;
	}
	
	
	/**
	 * Tests that a new pin can be set for an account
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public Boolean testPinSetting() throws RemoteCodeException {
		ActorName context = (ActorName)initStdTest("Pin Setting");
		call(context, "getNewTestContext");
		ActorName theBank = (ActorName)call(context,"getBank");
		ActorName theATM = (ActorName)call(context, "getATM");
		Integer acctID = (Integer)call(context, "getFirstAccount");
		Integer acctPin = (Integer)call(context, "getFirstPin"); 
			
		final int withdrawAmount = 40;
		final int newPin = 3333;
		
		BankingResponse pinResponse = (BankingResponse) call(theATM, "setPin", acctID, newPin, acctPin);
		BankingResponse withdrawResponse = (BankingResponse) call(theATM, "withdraw", acctID, withdrawAmount, newPin);
		boolean success = (pinResponse instanceof SuccessResponse);
	
		printTestResult("Pin Setting", success);
		return success;
	}
}
