/*
 * Authors:
 * 
 * Jeffrey Dennhardt
 * Kurt Hendle
 * Yuzuko Nakamura
 * Jurand Nogiec
 * 
 * Written Fall 2009
 */

package edu.illinois.cs427.Banking;

import java.util.Vector;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

public class ATM extends Actor {
	private static final long serialVersionUID = -5594745659933374035L;

	public ActorName localBank = null;
	private Vector<ActorName> banks = null;

	public ATM(ActorName bank) {
		this.localBank = bank;
	}
	
	/**
	 * There is a bug, we put a dummy value i so that
	 * the system can differentiate between this constructor
	 * and the one above.
	 * 
	 * @param i
	 * @param bankList
	 */
	public ATM(int i, Vector<ActorName> bankList) {
		this.banks = bankList;
	}

	/**
	 * makes an ATM know which bank it is communicating with
	 * 
	 * @param bankActorName
	 * @throws RemoteCodeException
	 */
	/*@message
	public void assignBank(ActorName bankActorName) throws RemoteCodeException {
		this.localBank = bankActorName;
	}*/

	/**
	 * deposit method a customer would call using an ATM (messages the bank)
	 * 
	 * @param accountName
	 * @param amount
	 */
	@message
	public Response deposit(String accountName, Double amount)
			throws RemoteCodeException {
		if(banks != null){
			localBank=banks.get(determineBank(accountName));
		}
		Response newBal = (Response) call(localBank, "deposit", accountName, amount);
		return newBal;
	}

	/**
	 * withdraw method a customer would call using an ATM (messages the bank)
	 * 
	 * @param accountName
	 * @param amount
	 * @return
	 */
	@message
	public Response withdraw(String accountName, Double amount)
			throws RemoteCodeException {
		if(banks != null)
			localBank=banks.get(determineBank(accountName));
		Response newBal = (Response) call(localBank, "withdraw", accountName, amount);
		return newBal;
	}

	/**
	 * transfer method a customer would call using an ATM (messages the bank)
	 * 
	 * @param accountName
	 * @param amount
	 */
	@message
	public Response transfer(String acctNameFrom, String acctNameTo, Double amount)
			throws RemoteCodeException {
		Response didWork = (Response) call(localBank, "transfer", acctNameFrom, acctNameTo, amount);
		return didWork;
	}
	
	/**
	 * transfer method used in a scalable design for when multiple banks are involved
	 * in a transfer oepration
	 * 
	 * @param acctNameFrom
	 * @param acctNameTo
	 * @param amount
	 */
	@message
	public Response scalableTransfer(String acctNameFrom, String acctNameTo, Double amount)
		throws RemoteCodeException {
		ActorName toBank;
		localBank = banks.get(determineBank(acctNameFrom));
		toBank = banks.get(determineBank(acctNameTo));
		Response didWork = (Response) call(localBank, "scalableTransfer", acctNameFrom, acctNameTo,
				toBank, amount);
		return didWork;
	}

	/**
	 * calls a balance inquiry at the bank and returns the balance
	 * 
	 * @param accountName
	 * @return balance of the account
	 * @throws RemoteCodeException
	 */
	@message
	public Response balanceInquiry(String accountName) throws RemoteCodeException {
		if(banks != null)
			localBank=banks.get(determineBank(accountName));
		return (Response) call(localBank, "balanceInquiry", accountName);
	}
	
	//method for determining the bank which an account is housed at
	private int determineBank(String accountNumber) {
		int numBanks = banks.size();
		int toDivide = 100000/numBanks;
		int theBank = Integer.parseInt(accountNumber)/toDivide;
		return theBank;
	}
}
