/*
 * Authors:
 * 
 * Jeffrey Dennhardt
 * Kurt Hendle
 * Yuzuko Nakamura
 * Jurand Nogiec
 * 
 * Written Fall 2009
 */

package edu.illinois.cs427.Banking;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

public class BootActor extends Actor {
	private static final long serialVersionUID = -17078175217038003L;

	private ArrayList<String> accounts = new ArrayList<String>();

	// deposit test -- make 1 bank, add account, deposit funds
	@message
	public void normalDepositTest_0() throws RemoteCodeException,
			InterruptedException {
		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		Response balanceResponse = (Response) call(atm, "deposit", accounts
				.get(0), 10.40);
		balanceResponse = (Response) call(atm, "deposit", accounts.get(0), 10.0);

		assert ((Boolean) balanceResponse.isSuccessful() && (Double) balanceResponse
				.getBalance() == 20.40);
	}

	// deposit test -- make multiple ATMs, 1 bank, add account, deposit funds
	@message
	public void twoDepositsTest_1() throws RemoteCodeException,
			InterruptedException {
		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);
		ActorName atm2 = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());


		Response balanceResponse1 = (Response) call(atm, "deposit", accounts
				.get(0), 10.40);
		Response balanceResponse2 = (Response) call(atm2, "deposit", accounts
				.get(0), 7.65);

		assert ((Boolean) balanceResponse2.isSuccessful()
				&& (Double) balanceResponse2.getBalance() == 18.05
				&& (Boolean) balanceResponse1.isSuccessful() && (Double) balanceResponse1
				.getBalance() == 10.40);
	}

	// normal withdraw
	@message
	public void normalWithdrawTest_2() throws RemoteCodeException,
			InterruptedException {
		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		Response balanceResponse = (Response) call(atm, "deposit", accounts
				.get(0), 10.40);
		balanceResponse = (Response) call(atm, "withdraw", accounts.get(0),
				10.40);

		assert ((Boolean) balanceResponse.isSuccessful() && (Double) balanceResponse
				.getBalance() == 0.0);
	}

	// withdraw without enough balance.
	@message
	public void insufficientFundsTest_3() throws RemoteCodeException,
			InterruptedException {
		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		Response balanceResponse = (Response) call(atm, "deposit", accounts.get(0), 10.40);
		balanceResponse = (Response) call(atm, "withdraw", accounts.get(0), 11.40);

		assert ((Boolean) balanceResponse.isSuccessful() && (Double) balanceResponse
				.getBalance() == 10.40);

	}

	// two withdraws
	@message
	public void twoWithdrawsTest_4() throws RemoteCodeException,
			InterruptedException {

		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);
		ActorName atm2 = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		call(atm, "deposit", accounts.get(0), 25.00);

		Response balanceResponse1 = (Response) call(atm, "withdraw", accounts.get(0),
				10.00);
		Response balanceResponse2 = (Response) call(atm2, "withdraw",
				accounts.get(0), 7.00);

		assert ((Boolean) balanceResponse2.isSuccessful()
				&& (Double) balanceResponse2.getBalance() == 8.00
				&& (Boolean) balanceResponse1.isSuccessful() && (Double) balanceResponse1
				.getBalance() == 15.00);
	}
	

	// transfer test
	@message
	public void basicTransferTest_5() throws RemoteCodeException,
			InterruptedException {

		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());
		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		call(atm, "deposit", accounts.get(0), 10.01);
		call(atm, "transfer", accounts.get(0), accounts.get(1), 1.01);

		Response senderResponse = (Response) call(atm, "balanceInquiry", accounts
				.get(0));
		Response recipientResponse = (Response) call(atm, "balanceInquiry", accounts
				.get(1));

		assert ((Boolean) senderResponse.isSuccessful()
				&& (Double) senderResponse.getBalance() == 9.0
				&& (Boolean) recipientResponse.isSuccessful() && (Double) recipientResponse
				.getBalance() == 1.01);
	
	}

	// see if unique account numbers are actually being made.
	@message
	public void uniqueAccountNumberTest_6() throws RemoteCodeException {
		ActorName bank = create(Bank.class);
		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());
		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		String acctNr1 = accounts.get(0);
		String acctNr2 = accounts.get(1);

		assert (!acctNr1.equals(acctNr2));
	}

	// test the return of a balance inquiry after a string of operations
	@message
	public void balanceInquiryTest_7() throws RemoteCodeException {
		ActorName bank = create(Bank.class);
		ActorName atm1 = create(ATM.class, bank);
		ActorName atm2 = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());


		String account = accounts.get(0);

		Response accountResponse = (Response) call(atm1, "deposit", account, 150.57);
		accountResponse = (Response) call(atm2, "deposit", account, 0.43);

		assert ((Boolean) accountResponse.isSuccessful() && (Double) accountResponse
				.getBalance() == 151.00);
		
		accountResponse = (Response) call(atm1, "withdraw", account, 51.00);

		assert ((Boolean) accountResponse.isSuccessful() && (Double) accountResponse
				.getBalance() == 100.00);
	}

	//close account test
	@message
	public void normalCloseAccount_8() throws RemoteCodeException {
		ActorName bank = create(Bank.class);
		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());
		Response isClosed = (Response) call(bank, "closeAccount", accounts.get(0));

		assert (isClosed.isSuccessful());
	}

	//close account with nonzero balance
	@message
	public void closeAccountNonZero_9() throws RemoteCodeException {
		ActorName bank = create(Bank.class);
		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());
		ActorName atm1 = create(ATM.class, bank);
		String account = accounts.get(0);

		call(atm1, "deposit", account, 100.0);

		Response isClosed = (Response) call(bank, "closeAccount", accounts.get(0));

		assert !isClosed.isSuccessful();
	}

	//transfer with insufficient funds
	@message
	public void transferInsufficientFunds_10() throws SecurityException,
			RemoteCodeException {
		ActorName bank = create(Bank.class);
		ActorName atm = create(ATM.class, bank);

		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());
		accounts.add(((Response) call(bank, "addAccount")).getAcctNr());

		call(atm, "deposit", accounts.get(0), 10.01);
		call(atm, "transfer", accounts.get(0), accounts.get(1), 11.01);

		Response senderResponse = (Response) call(atm, "balanceInquiry", accounts
				.get(0));
		Response recipientResponse = (Response) call(atm, "balanceInquiry", accounts
				.get(1));

		assert ((Boolean) senderResponse.isSuccessful()
				&& (Double) senderResponse.getBalance() == 10.01
				&& (Boolean) recipientResponse.isSuccessful() && (Double) recipientResponse
				.getBalance() == 0.0);
	}
	
	/**
	 * Creates list of banks
	 * Creates list of ATMs, using list of banks
	 * Using ATM 5, deposit 10.01 into account 0 (The bank used will depend on the account number)
	 * Then, using ATM 8, it will get the account balance from account 0
	 * @throws SecurityException
	 * @throws RemoteCodeException
	 */
	@message
	public void scalableTestSimpleDeposit_11() throws SecurityException, RemoteCodeException {
		Vector<ActorName> bankList = generateBanks(10);
		Vector<ActorName> atmList = generateATMs(bankList, 10);
		
		accounts.add(((Response) call(bankList.get(2), "scalableAddAccount")).getAcctNr());
		call(atmList.get(5), "deposit", accounts.get(0), 10.01);
		Response senderResponse = (Response) call(atmList.get(8), "balanceInquiry", accounts.get(0));
		assert senderResponse.getBalance() == 10.01;
	}
	
	/**
	 * Create list of 100 banks
	 * create list of 10 ATMs, with list of Banks given to each
	 * ATM 5 is used to deposit 10.01 into each of 20 accounts created
	 * balance inquiries made on each of 20 accounts through ATM 8 and balance printed
	 * @throws SecurityException
	 * @throws RemoteCodeException
	 */
	@message
	public void scalableTestMultipleDeposits_12() throws SecurityException, RemoteCodeException {
		Vector<ActorName> bankList = generateBanks(100);
		Vector<ActorName> atmList = generateATMs(bankList, 10);
		
		for(int ind=0; ind<20; ind++){
			accounts.add(((Response) call(bankList.get(ind), "scalableAddAccount")).getAcctNr());
		}
		
		for(int ind1=0; ind1<20; ind1++){
			call(atmList.get(5), "deposit", accounts.get(ind1), 10.50);
		}
		
		Response senderResponse = null;
		for(int ind2=0; ind2<20; ind2++){
			senderResponse = (Response) call(atmList.get(8), "balanceInquiry", accounts.get(ind2));
			assert senderResponse.getBalance() == 10.50;
		}	
	}
	
	/**
	 * Tests a single scalable withdrawal after depositing 10.01 into a single account
	 * using different ATMs.
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void scalableTestWithdraw_13() throws RemoteCodeException {
		Vector<ActorName> bankList = generateBanks(10);
		Vector<ActorName> atmList = generateATMs(bankList, 10);
		
		accounts.add(((Response) call(bankList.get(5), "scalableAddAccount")).getAcctNr());
		call(atmList.get(5), "deposit", accounts.get(0), 10.01);
		call(atmList.get(3), "withdraw", accounts.get(0), 5.01);
		Response senderResponse = (Response) call(atmList.get(8), "balanceInquiry", accounts.get(0));
		
		assert senderResponse.getBalance() == 5.00;
	}
	
	/**
	 * Tests withdraws on each account when multiple accounts exist in the system 
	 * at different banks
	 * @throws SecurityException
	 * @throws RemoteCodeException
	 */
	@message
	public void scalableTestMultipleWithdraw_14() throws SecurityException, RemoteCodeException {
		Vector<ActorName> bankList = generateBanks(20);
		Vector<ActorName> atmList = generateATMs(bankList, 10);
		
		for(int ind=0; ind<20; ind++){
			accounts.add(((Response) call(bankList.get(ind), "scalableAddAccount")).getAcctNr());
		}
		
		for(int ind1=0; ind1<20; ind1++){
			call(atmList.get(ind1%10), "deposit", accounts.get(ind1), 10.01);
		}
		
		for(int index=0; index<20; index++)
			call(atmList.get(index%10), "withdraw", accounts.get(index), 5.01);
		
		for(int ind2=0; ind2<20; ind2++){
			Response senderResponse = (Response) call(atmList.get(ind2%10), "balanceInquiry", accounts.get(ind2));
			assert senderResponse.getBalance() == 5.00;
		}	
	}
	
	
	/**
	 * Tests the scalable transfer methods which handle the case that a transfer involves
	 * multiple banks
	 * @throws RemoteCodeException 
	 * @throws SecurityException 
	 */
	@message
	public void scalableTestTransfer_15() throws SecurityException, RemoteCodeException {
		Vector<ActorName> bankList = generateBanks(10);
		Vector<ActorName> atmList = generateATMs(bankList, 10);
		
		accounts.add(((Response) call(bankList.get(5), "scalableAddAccount")).getAcctNr());
		accounts.add(((Response) call(bankList.get(8), "scalableAddAccount")).getAcctNr());
		
		call(atmList.get(2), "deposit", accounts.get(0), 10.01);
		call(atmList.get(4), "deposit", accounts.get(1), 10.01);
		
		call(atmList.get(6), "scalableTransfer", accounts.get(0), accounts.get(1), 5.00);
		
		Response senderResponse = (Response) call(atmList.get(8), "balanceInquiry", 
				accounts.get(0));
		assert senderResponse.getBalance() == 5.01;
		
		senderResponse = (Response) call(atmList.get(0), "balanceInquiry", accounts.get(1));
		assert senderResponse.getBalance() == 15.01;
	}
	
	
	//************ private methods for generating bank and atm actors ***************/
	private Vector<ActorName> generateBanks(int numBanks) throws SecurityException, RemoteCodeException {
		Vector<ActorName> bankList = new Vector<ActorName>();
		for(int bankNum=0; bankNum<numBanks; bankNum++) {
			bankList.add(create(Bank.class, bankNum, numBanks));
		}
		
		return bankList;
	}
	
	
	private Vector<ActorName> generateATMs(Vector<ActorName> banks, int numberATMs) throws SecurityException, RemoteCodeException {
		Vector<ActorName> atmList = new Vector<ActorName>();
		
		for(int atmNum=0; atmNum<numberATMs; atmNum++){
			int dummyValueWTF = 0;
			atmList.add(create(ATM.class, dummyValueWTF, banks));
		}
		
		return atmList;
	}

}
