package osl.examples.sorting;

import java.util.Random;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.ActorRequest;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

public class QuicksortCopy extends Actor {
    private static final long serialVersionUID = -6367617467463969386L;

    private Random generator = new Random(999999);

	@message
	public void boot(Integer size) throws RemoteCodeException {
		int A[] = new int[size];
		for (int i = 0; i < size; i++)
			A[i] = generator.nextInt(1000000);

		//System.out.println("Before sorting: ");
		//for (int i = 0; i < size; i++)
		//	System.out.print(A[i] + " ");
		//System.out.println();

		ActorName sorter = create(QuicksortCopy.class);

		A = (int[])call(sorter, "sort", A);

		//System.out.println("After sorting: ");
		//for (int i = 0; i < size; i++) {
		//	if (i > 0 && A[i - 1] > A[i]) {
		//		System.out.println("Error!!!");
		//		break;
		//	}
		//	System.out.print(A[i] + " ");
		//}
		//System.out.println();
	}

	@message
	public int[] sort(int[] A) throws RemoteCodeException {
		if (A.length==0)
			return A;
		int f = 0, l = A.length - 1;
		int pivot_index = partition(A, f, l);
		
		// make copies of two arrays
		int[] B = new int [pivot_index-f], C = new int [l - pivot_index];
		for (int i = f; i < pivot_index; i++)
			B[i - f] = A[i]; 
		for (int i = pivot_index + 1; i <= l; i++)
			C[i - pivot_index - 1] = A[i]; 
		
		ActorName sorter1 = create(QuicksortCopy.class);
		B = (int[])call(sorter1, "sort", B);
		ActorName sorter2 = create(QuicksortCopy.class);
		C = (int[])call(sorter2, "sort", C);
		
		//merge step here
		for (int i = f; i < pivot_index; i++)
			A[i] = B[i - f]; 
		for (int i = pivot_index + 1; i <= l; i++)
			A[i] = C[i - pivot_index - 1];
		
		return A;
	}

	/** Catch the asynchronous exception. */
	@message
	public void asynchException(ActorRequest cause, Exception e) {
		e.printStackTrace();
	}
	
	private int partition(int A[], int f, int l) {
		int pivot = A[f];
		int low = f + 1;

		for (int i = low; i <= l; i++) {
			if (A[i] <= pivot) {
				swap(A, i, low);
				low++;
			}
		}
		swap(A, f, low - 1);
		return low - 1;
	}

	private void swap(int A[], int x, int y) {
		if (x == y)
			return;
		int temp = A[x];
		A[x] = A[y];
		A[y] = temp;
	}


}
