package rainbow_fish.project;

import osl.manager.Actor;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

/**
 * AccountActor object represents an account in the bank system.
 * TransactionActors perform transactions with AccountActors.
 */
public class AccountActor extends Actor {

	private static final long serialVersionUID = 1030595902702952246L;

	private Amount balance = Amount.ZERO_AMOUNT;

	/**
	 * Returns an <tt>Amount</tt> object representing the balance in this
	 * <tt>Account</tt>.
	 * 
	 * @return <tt>Amount</tt> of this.
	 * @throws RemoteCodeException
	 */
	@message
	public Amount getBalance() throws RemoteCodeException {
		return balance;
	}

	/**
	 * Deposits the value specified by the amount <tt>Amount</tt> parameter into
	 * this <tt>Account</tt>.
	 * 
	 * @param amount
	 *            An <tt>Amount</tt> representing the value to be deposited
	 * @throws RemoteCodeException
	 */
	@message
	public void deposit(Amount amount) throws RemoteCodeException {
		balance = balance.add(amount);
	}

	/**
	 * Withdraws the value specified by the amount <tt>Amount</tt> parameter
	 * from this <tt>Account</tt>. Returns a 1 <tt>Integer</tt> object if
	 * successful, -1 otherwise (when balance is less than withdraw amount).
	 * 
	 * @param amount
	 *            An <tt>Amount</tt> object representing the value to withdraw
	 *            from <tt>this</tt>
	 * @return An <tt>Integer</tt> object, 1 if successful, -1 if failed.
	 * @throws RemoteCodeException
	 */
	@message
	public Integer withdraw(Amount amount) throws RemoteCodeException {
		if (balance.compareTo(amount) == -1) {
			return new Integer(-1);
		} else {
			balance = balance.subtract(amount);
			return new Integer(1);
		}
	}
}
