package rainbow_fish.project;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;
import rainbow_fish.project.*;

public class __TestATMActor extends Actor {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8224635902702957246L;

	/**
	 * Call all the tests inside this class
	 * @throws RemoteCodeException
	 */
	@message
	public void testAll() throws RemoteCodeException {
		testDepositWithdraw();
		testTransfer();
	}

	/**
	 * Test deposit 100.00 and withdraw 50.00 concurrently.
	 * @throws RemoteCodeException
	 */
	@message
	public void testDepositWithdraw() throws RemoteCodeException {
		ActorName bank = create(BankActor.class);
		String id = (String) call(bank, "openAccount");
		Response afterOpen = (Response) call(bank, "accountExists", id);
		if (afterOpen instanceof FailureResponse)
			System.out.println(((FailureResponse) afterOpen).getMessage());

		ActorName atm1 = create(ATMActor.class, bank);
		ActorName atm2 = create(ATMActor.class, bank);
		send(atm1, "deposit", id, new Amount(100.00));
		call(atm2, "withdraw", id, new Amount(50.00));

		Response balanceResponse = (Response) call(atm1, "getBalance", id);

		// balance1 is the balance if withdraw is processed first
		Amount balance1 = new Amount(50.00);
		// balance2 is the balance if deposit is processed first
		Amount balance2 = new Amount(50.00);
		Amount balance3 = new Amount(0.00);
		// If a balance was returned
		if (balanceResponse instanceof BalanceResponse) {
			Amount returnValue = ((BalanceResponse) balanceResponse)
					.getBalance();
			// If balance = 100 or 50 or 0
			if (returnValue.equals(balance1) || returnValue.equals(balance2)
					|| returnValue.equals(balance3)) {
				System.out
						.println("success in concurrent deposit and withdraw");
				System.out.println("amount : " + returnValue.toString());
			}

			else {
				System.err
						.println("incorrect amount is returned in getBalance in concurrent deposit and withdraw");
				System.err.println("amount : " + returnValue.toString());
			}
		}

		else {
			System.err
					.println("getBalance fail in concurrent deposit and withdraw");
		}

	}

	/**
	 * Test transfer 50.00 from primary account with initial balance of 100.00 
	 * to secondary account with initial balance of 0 and transfer 50.00 from
	 * the same secondary account to primary account concurrently.
	 * @throws RemoteCodeException
	 */
	@message
	public void testTransfer() throws RemoteCodeException {
		ActorName bank = create(BankActor.class);

		// open primary account to use as a source account for the transfer
		String primaryAccount = (String) call(bank, "openAccount");
		Response afterOpen1 = (Response) call(bank, "accountExists",
				primaryAccount);
		if (afterOpen1 instanceof FailureResponse)
			System.out.println(((FailureResponse) afterOpen1).getMessage());

		// open secondary account to use as the destination account for the
		// transfer
		String secondaryAccount = (String) call(bank, "openAccount");
		Response afterOpen2 = (Response) call(bank, "accountExists",
				secondaryAccount);
		if (afterOpen2 instanceof FailureResponse)
			System.out.println(((FailureResponse) afterOpen2).getMessage());

		ActorName atm1 = create(ATMActor.class, bank);
		ActorName atm2 = create(ATMActor.class, bank);
		Response response = (Response) call(atm1, "deposit", primaryAccount,
				new Amount(100.00));
		send(atm1, "transfer", primaryAccount, secondaryAccount, new Amount(
				50.00));
		call(atm2, "transfer", secondaryAccount, primaryAccount, new Amount(
				50.00));
		
		/**
		 * NOTE : Tests should never fail, use Basset to check.
		 */
	}

}
