package rainbow_fish.project;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

public class __TestAccountActor extends Actor {

	private static final long serialVersionUID = 8224635902702957246L;

	/**
	 * Call all the methods in the __TestAccountActor class
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testAll() throws RemoteCodeException {
		testGetBalance();
		testDeposit();
		testWithdraw();
	}

	/**
	 * Test if the original balance of an account is set to zero
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testGetBalance() throws RemoteCodeException {

		ActorName a1 = create(AccountActor.class);
		Amount result = (Amount) call(a1, "getBalance");
		if (!result.equals(new Amount(0.00))) {
			System.out.println("getBalance() fail in testGetBalance()");
		} else {
			System.out.println("TestAccountActor.getBalance() success");
		}
	}

	/**
	 * Test for successful deposit
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testDeposit() throws RemoteCodeException {
		ActorName a1 = create(AccountActor.class);
		call(a1, "deposit", new Amount(100.00));
		Amount result = (Amount) call(a1, "getBalance");
		if (!result.equals(new Amount(100.00))) {
			System.out.println("deposit() fail in testDeposit()");
		} else {
			System.out.println("TestAccountActor.testDeposit() success");
		}
	}

	/**
	 * Test for invalid withdraw and valid withdraw
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testWithdraw() throws RemoteCodeException {

		ActorName a1 = create(AccountActor.class);
		call(a1, "deposit", new Amount(100.00));
		Integer result = (Integer) call(a1, "withdraw", new Amount(200.00));
		Amount amount = (Amount) call(a1, "getBalance");

		if (!amount.equals(new Amount(100.00))) {
			System.out
					.println("withdraw() fail on overdraft in testWithdraw()");
		} else {
			System.out.println("TestAccountActor.testWithdraw() success");
		}

		Integer result2 = (Integer) call(a1, "withdraw", new Amount(100.00));
		amount = (Amount) call(a1, "getBalance");
		if (!amount.equals(Amount.ZERO_AMOUNT)) {
			System.out
					.println("withdraw() fail on valid withdraw in testWithdraw()");
		} else {
			System.out.println("TestAccountActor.testWithdraw() success");
		}
	}
}
