package rainbow_fish.project;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;
import rainbow_fish.project.*;

public class __TestCompare extends Actor {

	/**
	 * This test is designed to emulate synchronous transfers in threads.
	 * Used to compare runtime of this on Basset, and threads on JPF.
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testCompare() throws RemoteCodeException {

		ActorName bank = create(BankActor.class);

		// open primary account to use as a source account for the transfer
		String primaryAccount = (String) call(bank, "openAccount");
		Response afterOpen1 = (Response) call(bank, "accountExists",
				primaryAccount);
		if (afterOpen1 instanceof FailureResponse)
			System.out.println(((FailureResponse) afterOpen1).getMessage());

		// open secondary account to use as the destination account for the
		// transfer
		String secondaryAccount = (String) call(bank, "openAccount");
		Response afterOpen2 = (Response) call(bank, "accountExists",
				secondaryAccount);
		if (afterOpen2 instanceof FailureResponse)
			System.out.println(((FailureResponse) afterOpen2).getMessage());

		ActorName atm1 = create(ATMActor.class, bank);
		ActorName atm2 = create(ATMActor.class, bank);
		Response response1 = (Response) call(atm1, "deposit", primaryAccount,
				new Amount(100.00));
		Response response2 = (Response) call(atm2, "deposit", secondaryAccount,
				new Amount(100.00));

		send(atm1, "transfer", primaryAccount, secondaryAccount, new Amount(
				50.00));
		send(atm2, "transfer", secondaryAccount, primaryAccount, new Amount(
				50.00));

	}

}