package rainbow_fish.project;

import osl.manager.Actor;
import osl.manager.ActorName;
import osl.manager.RemoteCodeException;
import osl.manager.annotations.message;

public class __TestTransactionActor extends Actor {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8224635902702957247L;

	/**
	 * Call all the tests inside this class
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testAll() throws RemoteCodeException {
		testDeposit();
		testWithdraw();
		testTransfer();
	}

	/**
	 * Test deposit 100.00 into an account
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testDeposit() throws RemoteCodeException {
		ActorName account = create(AccountActor.class);
		ActorName trans = create(rainbow_fish.project.TransactionActor.class,
				account);

		Response responseObject = (Response) call(trans, "deposit", new Amount(
				100.00));
		Amount result = (Amount) call(account, "getBalance");

		if (!result.equals(new Amount(100.00))) {
			System.out
					.println("TestTransactionActor.testDeposit() fail in testDeposit()");
		} else {
			System.out.println("TestTransactionActor.testDeposit() success");
		}
	}

	/**
	 * Test valid and invalid withdraw into an account with initial balance of
	 * 100
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testWithdraw() throws RemoteCodeException {
		ActorName account = create(AccountActor.class);
		ActorName trans = create(TransactionActor.class, account);

		Response responseDepositObject = (Response) call(trans, "deposit",
				new Amount(100.00));
		Response responseWithdrawOverdraftObject = (Response) call(trans,
				"withdraw", new Amount(200.00));
		Amount amount = (Amount) call(account, "getBalance");

		if (!(responseWithdrawOverdraftObject instanceof FailureResponse)
				|| !amount.equals(new Amount(100.00))) {
			System.out
					.println("TestTransactionActor.testWithdraw() fail in testWithdraw()");
		} else {
			Response responseWithdrawObject = (Response) call(trans,
					"withdraw", new Amount(100.00));
			amount = (Amount) call(account, "getBalance");
			if (responseWithdrawObject instanceof FailureResponse
					|| !amount.equals(Amount.ZERO_AMOUNT)) {
				System.out
						.println("TestTransactionActor.testWithdraw() fail on valid withdraw in testWithdraw()");
			} else {
				System.out
						.println("TestTransactionActor.testWithdraw() success");
			}
		}
	}

	/**
	 * Test transfer 100from an account with initial balance of 100.00 to
	 * another account with zero balance
	 * 
	 * @throws RemoteCodeException
	 */
	@message
	public void testTransfer() throws RemoteCodeException {
		ActorName primaryAccount = create(AccountActor.class);
		ActorName secondaryAccount = create(AccountActor.class);
		ActorName trans = create(TransactionActor.class, primaryAccount,
				secondaryAccount);

		Response responseDepositObject = (Response) call(trans, "deposit",
				new Amount(100.00));
		Response responseTransferObject = (Response) call(trans, "transfer",
				new Amount(100.00));

		Amount amount1 = (Amount) call(primaryAccount, "getBalance");
		Amount amount2 = (Amount) call(secondaryAccount, "getBalance");

		if (responseTransferObject instanceof FailureResponse
				|| (!amount1.equals(Amount.ZERO_AMOUNT) && !amount2
						.equals(new Amount(100.00)))) {
			System.out
					.println("TestTransactionActor.testTransfer() fail on overdraft in testWithdraw()");
		} else {
			System.out.println("TestTransactionActor.testTransfer() success");
		}
	}
}
